import os, pygame

from math import sqrt
from string import Template, atoi

from pygame.locals import *
import pygame.surfarray

FACING_UP = 0
FACING_DOWN = 1
FACING_LEFT = 2
FACING_RIGHT = 3

POINT_NODES = ["bigtree", "smalltree", "player", "pinetree", "plant1", "plant2", "plant3", "plant4", "hedge", "house", "ball", "caw", "bug", "kid", "gkid", "npc", "dollar", "dime", "penny", "hog", "sign", "heart", "heartframe","clock", "secretitem", "shopitem", "invitem", "boulder", "stump", "ladder", "mastercane"]
REGION_NODES = ["grass", "dirt", "dirtll", "dirtlr",
                "dirtul", "dirtur", "grassd", "grassu", "grassl", "grassr",
                "grassll", "grasslr", "grassul", "grassur", "exit", "secretexit", "water"]

NPC_POINTS = ["npc", "sign"]
ENEMY_POINTS = ["caw", "kid", "gkid", "hog"]
BLOCKING_POINTS = ["bigtree", "smalltree", "pinetree", "plant2", "plant3", "plant4", "ball", "house", "fence", "stump"]
GOLDS_POINTS = ["dollar", "dime", "penny"]
BLOCKING_REGIONS = []

GOLDS_AMOUNT = {
                "dollar" : 1.00,
                "dime" : 0.10,
                "penny" : 0.01
}

ITM_NET = "net"
ITM_SHOVEL = "shovel"
ITM_GLOVE = "glove"
ITM_AXE = "axe"
ITM_POTION = "potion"
ITM_HOE = "hoe"
ITM_LADDER = "ladder"

IMAGE_FILES = {
               "player" : "char/walk2.png",
               "rkid" : "char/rkid1.png",
               "hog" : "groundhog1.png",
               "dirt" : "dirt.png",
               "grass" : "grass.png",
               "dirtll" : "dirtlowerleftc.png",
               "dirtlr" : "dirtlowerrightc.png",
               "dirtul" : "dirtupperleftc.png",
               "dirtur" : "dirtupperrightc.png",
               "grassd" : "grassdown.png",
               "grassu" : "grassup.png",
               "grassl" : "grassleft.png",
               "grassr" : "grassright.png",
               "grassll" : "grasslowerleftc.png",
               "grasslr" : "grasslowerrightc.png",
               "grassul" : "grassupperleftc.png",
               "grassur" : "grassupperrightc.png",
               "bigtree" : "bigtree.png",
               "smalltree" : "smalltree.png",
               "house" : "house.png",
               "pinetree" : "pinetree.png",
               "plant1" : "plant1.png",
               "plant2" : "plant2.png",
               "plant3" : "plant3.png",
               "plant4" : "plant4.png",
               "ball" : "ball.png",
               "hedge" : "hedge.png",
               "caw" : "caw1.png",
               "bug" : "bug.png",
               "kid" : "char/kid1.png",
               "gkid" : "char/gkid1.png",
               "npc" : "shopkeeper.png",
               "sign" : "sign.png",
               "exit" : "hedge.png",
               "dollar" : "item/buxx.png",
               "dime" : "item/coin.png",
               "penny" : "item/coin2.png",
               "heart" : "item/heartfind.png",
               "heartframe" : "item/heartcontainer.png",
               "clock" : "item/clock.png",
               "secretitem" : "item/clock.png",
               "secretexit" : "item/clock.png",
               "shopitem" : "item/clock.png",
               "invitem" : "item/clock.png",
               "glove" : "item/itemselectglove.png",
               "net" : "item/itemselectnet.png",
               "shovel" : "item/itemselectshovel.png",
               "potion" : "item/gpotion.png",
               "axe" : "item/itemselectaxe.png",
               "boulder" : "boulder1.png",
               "stump" : "stump.png",
               "bighole" : "bighole.png",
               "ladder" : "ladder.png",
               "water" : "water1.png",
               "mastercane" : "mastercane.png",
              }

def colorize(pic, color):
    picarray  = pygame.surfarray.array3d(pic)
    for i in range(len(picarray)):
       for j in range(len(picarray[i])):
            for k in range(3):
                if (picarray[i][j][k] != 0 and picarray[i][j][k] != 255):
                    num = picarray[i][j][k] + color[k]
                    if num < 0:
                        num = 0
                    elif num >  255:
                        num = 255
                    picarray[i][j][k] = num
    return pygame.surfarray.make_surface(picarray)

def toggle_fullscreen():
    screen = pygame.display.get_surface()
    tmp = screen.convert()
    caption = pygame.display.get_caption()
    
    w,h = screen.get_width(),screen.get_height()
    flags = screen.get_flags()
    bits = screen.get_bitsize()
    
    pygame.display.quit()
    pygame.display.init()
    
    screen = pygame.display.set_mode((w,h),flags^FULLSCREEN,bits)
    screen.blit(tmp,(0,0))
    pygame.display.set_caption(*caption)

    pygame.mouse.set_visible(0) 
    pygame.key.set_mods(0) #HACK: work-a-round for a SDL bug??
    
    return screen

def load_image(name, colorkey=None):
    fullname = os.path.join('img', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

